import React, { Component } from 'react';
import {
  Input, 
  Empty,
  Checkbox
} from 'antd';
import Base from '@icode-run/component-layout/utils/Base';
import styles from './index.module.less';
import {
  CheckOutlined,
  PlusCircleOutlined, 
  MinusCircleOutlined,
  UpCircleOutlined,
} from '@ant-design/icons'

export default class MyInput extends Component {

  state = {
    enumOptions: [],
    activeItem: {},
    errors: {},
    currEnumOptionItem: {},
    selectValues: [],
  };

  static getDerivedStateFromProps({
    enumOptions, 
    selectValues
  }, prevState) {
    const options = {};
    if (enumOptions) {
      options.enumOptions = enumOptions.map(item => {
        return {
          label: item.label,
          value: item.value + ''
        }
      });
    }
    if (selectValues) {
      options.selectValues = selectValues;
    }
    return options;
  }

  componentDidMount() {

  }

  onSelect = (activeItem) => {
    const {onSelect, multiple} = this.props;
    const {selectValues = []} = this.state;
    const selectItemsObj = {};
    selectValues.forEach((value, index) => {
      selectItemsObj[value] = index;
    })

    
    if(selectItemsObj[activeItem.value] >= 0) {
      selectValues.splice(selectItemsObj[activeItem.value], 1);
    }
    else {
      selectValues.push(activeItem.value);
    }
    
    let newSelectValues = [].concat(selectValues);
    if(!multiple && newSelectValues.length > 1) {
      newSelectValues = [newSelectValues[newSelectValues.length - 1]];
    }

    this.setState({
      activeItem,
      selectValues: newSelectValues,
    })
    if(onSelect && typeof onSelect === 'function') {
      onSelect(newSelectValues);
    }
  }

  onRemove = (item, index) => {
    const {enumOptions = [], selectValues = []} = this.state;
    const {onSelect} = this.props;
    const selectIndex = selectValues.indexOf(item.value);
    enumOptions.splice(index, 1);
    if(selectIndex >= 0) {
      selectValues.splice(selectIndex, 1);
    }
    
    // this.setState({
    //   enumOptions,
    // });

    if(onSelect && typeof onSelect === 'function') {
      onSelect(selectValues);
    }
    this.onChange(enumOptions);
  }

  onMove = (item, index) => {
    const {enumOptions = []} = this.state;
    const newOption = JSON.parse(JSON.stringify(enumOptions[index]));

    enumOptions.splice(index, 1);
    enumOptions.splice(index - 1, 0, newOption);
    // this.setState({
    //   enumOptions,
    // });
    this.onChange(enumOptions);
  }

  onChange = (data) => {
    const {onChange} = this.props;
    if(onChange && typeof onChange === 'function') {
      onChange(data);
    }
  }

  onAdd = () => {
    const {currEnumOptionItem = {}, enumOptions = []} = this.state;

    const enumOptionsObj = {};
    enumOptions.map(item => {
      enumOptionsObj[item.value] = item;
    });
    const newOption = currEnumOptionItem;

    const errors = {};
    if(!newOption.label) {
      errors.label = '不能为空';
    }
    if(!newOption.value && newOption.value !== 0) {
      errors.value = '不能为空';
    }

    if(Object.values(errors).length) {
      this.setState({
        errors,
      })
      return;
    }

    enumOptionsObj[newOption.value] = newOption;

    const newEnumOptions = Object.values(enumOptionsObj);

    this.setState({
      enumOptions: newEnumOptions,
      currEnumOptionItem: {},
    });

    this.onChange(newEnumOptions);

  }

  render() {
    const {
      ableRemove, 
      ableMove, 
      ableAdd, 
      outputType = '', 
      optionsEmpty = '',
      size,
      ableChecked,
      onSelect
    } = this.props;
    const {
      enumOptions = [], 
      activeItem = {}, 
      errors, 
      currEnumOptionItem, 
      selectValues = []
    } = this.state;

    const selectItemsObj = {};
    selectValues.forEach(value => {
      selectItemsObj[value] = true;
    })

    return <div className={`icode-enum-options-box ${styles.enumOptions} ${styles[size]}`}>
      {enumOptions.length ? <>
        <div className={`icode-enum-options-head ${styles.enumItem} ${styles.enumItemTh}`}>
          {ableChecked ? <p className={styles.checkBox}></p> : null}
          <p>别名</p>
          <p>值</p>
          <p></p>
        </div>
        <div className={`icode-enum-options-body ${styles.body}`}>
        {enumOptions.map((item, index) => {
          return <div 
            className={`
            ${styles.enumItem} 
            ${activeItem.value === item.value ? styles.activeItem : ''}`} 
            onClick={() => {
              if(!ableChecked) {
                return;
              }
              this.onSelect(item);
            }}
            style={ableChecked ? {cursor: 'pointer'} : {}}
          >
            {ableChecked ? <p className={styles.checkBox}>
              <Checkbox checked={selectItemsObj[item.value]}/>
            </p> : null}
            <p>{item.label}</p>
            <p>{item.value}</p>
            <p>
              {ableRemove ? <MinusCircleOutlined 
              title="删除选项"
              onClick={(e) => {
                e.preventDefault();
                e.stopPropagation();
                this.onRemove(item, index);
              }}
              /> : null}
              {ableMove && index > 0 ? <UpCircleOutlined 
                title="向上移动"
                onClick={(e) => {
                  e.preventDefault();
                  e.stopPropagation();
                  this.onMove(item, index);
                }}
              /> : null}
            </p>
            </div>;
          })
        }
        </div>
      </> : 
      <Empty image={Empty.PRESENTED_IMAGE_SIMPLE} description={optionsEmpty ? optionsEmpty : '暂无选项!'}/>
      }

      {ableAdd ? <div className={`icode-enum-options-foot ${styles.enumItem} ${styles.enumItemFt}`}>
      
        <p style={ableChecked ? {flex: 2.4} : {}}>
          <Input
            size={size}
            style={{width: '100%'}}
            placeholder={`选项别名...`}
            value={currEnumOptionItem.label || ''}
            onChange={(e) => {
              const oldOption = currEnumOptionItem;
              let value = e.target.value;
              const newOption = {
                ...oldOption,
                label: value
              };
              this.setState({
                currEnumOptionItem: newOption
              })
            }}
          />
        </p>
        <p style={{paddingLeft: '0px'}}>
          <Input
            size={size}
            style={{width: '100%'}}
            placeholder={`${outputType.indexOf('number') >= 0 ? '数字': ''}选项值...`}
            value={currEnumOptionItem.value || currEnumOptionItem.value === 0 ? currEnumOptionItem.value + '' : ''}
            onChange={(e) => {
              const oldOption = currEnumOptionItem;
              let value = e.target.value;
              //value = Number(value);
              if(outputType.indexOf('number') >= 0 && !value && value !== 0) {
                this.setState({
                  currEnumOptionItem: {
                    ...oldOption,
                    value: ''
                  }
                })
                return;
              }
              this.setState({
                currEnumOptionItem: {
                  ...oldOption,
                  value,
                }
              })
            }}
          />
        </p>
        <p>
          <PlusCircleOutlined  
            title="新增选项"
            onClick={this.onAdd}
          />
          
        </p>
      </div> : null}
    </div>;
  }
};

//<div>暂无选项! <br/> 如果为固定的几个值，请添加枚举选项！</div>
