import React, { Component } from 'react';
import {Select, InputNumber, Form} from 'antd';
import Base from '@icode-run/component-layout/utils/Base';
import Enum from './Enum';
import styles from './index.module.less';
import {getOutputValue} from '@icode-run/utils/es/dataType'

export default class MyComponent extends Base {
  state = {
    enumOptions: []
  }

  static getDerivedStateFromProps({
    selectedParma, 
    enumOptionsParam
  }, prevState) {
    const options = {};
    if (selectedParma) {
      options.selectValues = getOutputValue({
        outputType: '[string]', 
        value: selectedParma.value
      });
    }
    if(enumOptionsParam) {
      options.enumOptions = enumOptionsParam.value || [];
    }
    console.log('-----getDerivedStateFromProps', options);
    return options;
  }

  componentDidMount() {

  }

  onSelect = (values) => {
    const {selectedParma = {}} = this.props;
    if(selectedParma.onChange && typeof selectedParma.onChange === 'function') {
      console.log('-----onSelect', values);
      selectedParma.onChange(values && values.length ? values : '');
    }
  }

  onChangeEnumOptions = (options) => {
    const {enumOptionsParam = {}} = this.props;
    if(enumOptionsParam.onChange && typeof enumOptionsParam.onChange === 'function') {
      console.log('-----onChangeEnumOptions', options);
      enumOptionsParam.onChange(options);
    }
  }

  getEnumCheckMultiple = () => {
    const {selectedParma} = this.props;
    const outputType = selectedParma.outputType;
    let isMultiple = false;
    if(outputType === '[string]' || outputType === '[number]') {
      isMultiple = true;
    }
    return isMultiple;
  }

  render() {
    const {
      className, 
      style,
      ableRemove, 
      ableMove, 
      ableAdd, 
      size = 'middle', 
      optionsEmpty, 
      placeholder, 
      onlyEnumOptions,
      ableChecked,
      bordered
    } = this.props;
    const {enumOptions = [], selectValues = []} = this.state;
    return <div className={className || ''} style={style || {}}>
      {onlyEnumOptions ? <Enum
      ableRemove={ableRemove}
      ableMove={ableMove}
      ableAdd={ableAdd}
      ableChecked={ableChecked}
      multiple={this.getEnumCheckMultiple()}
      size={size}
      enumOptions={enumOptions}
      optionsEmpty={optionsEmpty}
      //outputType={'[{label, value}]'}
      selectValues={selectValues}
      onSelect={(selectValues, selectItems) => {
        this.onSelect(selectValues);
      }}
      onChange={(enumOptions) => {
        this.onChangeEnumOptions(enumOptions);
      }}
    /> : <Select
    className={`${styles.enumInput} ${styles[size]}`}
    style={{ width: "100%" }}
    size={size}
    bordered={bordered}
    value={selectValues}
    mode="multiple"
    dropdownRender={menu => (
      <Enum
        ableRemove={ableRemove}
        ableMove={ableMove}
        ableAdd={ableAdd}
        ableChecked={ableChecked}
        size={size}
        optionsEmpty={optionsEmpty}
        multiple={this.getEnumCheckMultiple()}
        //outputType={'[{label, value}]'}
        selectValues={selectValues}
        onSelect={(selectValues, selectItems) => {
          this.onSelect(selectValues);
        }}
        enumOptions={enumOptions}
        onChange={(enumOptions) => {
          this.onChangeEnumOptions(enumOptions);
        }}
      />
    )}
    options={[].concat(enumOptions)}
    placeholder={placeholder || '请先创建选项...'}
    onChange={(selectValues) => {
      this.onSelect(selectValues);
    }}
  >
  </Select>}
    </div>;
  }
};
